//+---------------------------------------------------------------------+
//|                                                     ImpulseOsMA.mq5 | 
//|                               Copyright  2008, Nikolay Dzencharski | 
//|                                                  dzenchar@gmail.com | 
//+---------------------------------------------------------------------+ 
//|          SmoothAlgorithms.mqh |
//|   (): __\\MQL5\Include        |
//+---------------------------------------------------------------------+ 
#property copyright "Copyright  2008, Nikolay Dzencharski"
#property link "dzenchar@gmail.com" 
//----   
#property version   "1.00"
//----     
#property indicator_separate_window 
//----    4
#property indicator_buffers 4 
//----     
#property indicator_plots   2

//+-----------------------------------+
//|     1 |
//+-----------------------------------+
//----   1   
#property indicator_type1   DRAW_FILLING
//----     
#property indicator_color1  clrBlueViolet,clrRed
//----    
#property indicator_label1  "Up Trend;Down Trend"
//+-----------------------------------+
//|     2 |
//+-----------------------------------+
//----      
#property indicator_type2 DRAW_COLOR_HISTOGRAM
//----      
#property indicator_color2 clrMagenta,clrBlue,clrTeal
//----   - 
#property indicator_style2 STYLE_SOLID
//----     2
#property indicator_width2 2
//----   
#property indicator_label2 "XMACD"

//+-----------------------------------+
//|          |
//+-----------------------------------+
#include <SmoothAlgorithms.mqh> 
//+-----------------------------------+

//----    CXMA   SmoothAlgorithms.mqh
CXMA XMA1,XMA2,XMA3,XMA4;
//+-----------------------------------+
//|             |
//+-----------------------------------+
enum Applied_price_ // 
  {
   PRICE_CLOSE_ = 1,     //Close
   PRICE_OPEN_,          //Open
   PRICE_HIGH_,          //High
   PRICE_LOW_,           //Low
   PRICE_MEDIAN_,        //Median Price (HL/2)
   PRICE_TYPICAL_,       //Typical Price (HLC/3)
   PRICE_WEIGHTED_,      //Weighted Close (HLCC/4)
   PRICE_SIMPL_,         //Simpl Price (OC/2)
   PRICE_QUARTER_,       //Quarted Price (HLOC/4) 
   PRICE_TRENDFOLLOW0_,  //TrendFollow_1 Price 
   PRICE_TRENDFOLLOW1_,  //TrendFollow_2 Price 
   PRICE_DEMARK_         //Demark Price
  };
/*enum Smooth_Method -     SmoothAlgorithms.mqh
  {
   MODE_SMA_,  //SMA
   MODE_EMA_,  //EMA
   MODE_SMMA_, //SMMA
   MODE_LWMA_, //LWMA
   MODE_JJMA,  //JJMA
   MODE_JurX,  //JurX
   MODE_ParMA, //ParMA
   MODE_T3,    //T3
   MODE_VIDYA, //VIDYA
   MODE_AMA,   //AMA
  }; */
//+-----------------------------------+
//|         |
//+-----------------------------------+
input Smooth_Method MACD_Method=MODE_EMA; //  
input uint Fast_XMA = 12; //  
input uint Slow_XMA = 26; //  
input  int MACD_Phase = 100;  //  ,
                       // JJMA    -100 ... +100,     ;
//  VIDIA   CMO,  AMA    
input Applied_price_ MACD_Price=PRICE_CLOSE_;//  MACD

input Smooth_Method Signal_Method=MODE_EMA; //   
input uint Signal_XMA=9; //   
input int Signal_Phase=100; //   ,
                            //   -100 ... +100,
//    ;


input Smooth_Method Ma_Method=MODE_SMA; //  
input uint Ma=12; // 
input int Ma_Phase=100; //  ,
                       // JJMA    -100 ... +100,     ;
//  VIDIA   CMO,  AMA    
input Applied_price_ Ma_Price=PRICE_CLOSE_;//  
//+-----------------------------------+
//----      
int min_rates_total,min_rates_1;
//----   ,    
//      
double XMACDBuffer[],ColorXMACDBuffer[],SignBuffer[],XSignBuffer[];
//+------------------------------------------------------------------+    
//| XMACD indicator initialization function                          | 
//+------------------------------------------------------------------+  
void OnInit()
  {
//----     
   min_rates_1=MathMax(XMA1.GetStartBars(MACD_Method,Fast_XMA,MACD_Phase),XMA1.GetStartBars(MACD_Method,Slow_XMA,MACD_Phase))+1;
   int min_rates_2=min_rates_1+XMA1.GetStartBars(Signal_Method,Signal_XMA,Signal_Phase);
   int min_rates_3=XMA1.GetStartBars(Ma_Method,Ma,Ma_Phase);
   min_rates_total=MathMax(min_rates_2,min_rates_3)+1;

//----      
   SetIndexBuffer(0,SignBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,XSignBuffer,INDICATOR_DATA);
   SetIndexBuffer(2,XMACDBuffer,INDICATOR_DATA);    
   SetIndexBuffer(3,ColorXMACDBuffer,INDICATOR_COLOR_INDEX);  
   
//----      
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,min_rates_total);
//----   ,      
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,EMPTY_VALUE);

//----      
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,min_rates_total);
//----   ,      
   PlotIndexSetDouble(1,PLOT_EMPTY_VALUE,EMPTY_VALUE);

//---           
   IndicatorSetString(INDICATOR_SHORTNAME,"ImpulseOsMA");
//---     
   IndicatorSetInteger(INDICATOR_DIGITS,0);
//----  
  }
//+------------------------------------------------------------------+  
//| XMACD iteration function                                         | 
//+------------------------------------------------------------------+  
int OnCalculate(
                const int rates_total,    //       
                const int prev_calculated,//       
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]
                )
  {
//----       
   if(rates_total<min_rates_total) return(0);

//----   
   int first,bar;
//----       
   double price,fast_xma,slow_xma,xmacd,sign_xma,curMA,cur,prev;
   static double prevMA;

//----     OnCalculate()
   if(prev_calculated>rates_total || prev_calculated<=0)//      
     {
      first=1;
     }
   else //      
     {
      first=prev_calculated-1;
     }

//----    
   for(bar=first; bar<rates_total && !IsStopped(); bar++)
     {
      price=PriceSeries(MACD_Price,bar,open,low,high,close);

      fast_xma=XMA1.XMASeries(1,prev_calculated,rates_total,MACD_Method,MACD_Phase,Fast_XMA,price,bar,false);
      slow_xma=XMA2.XMASeries(1,prev_calculated,rates_total,MACD_Method,MACD_Phase,Slow_XMA,price,bar,false);
      xmacd=(fast_xma-slow_xma)/_Point;
      sign_xma=XMA3.XMASeries(min_rates_1,prev_calculated,rates_total,Signal_Method,Signal_Phase,Signal_XMA,xmacd,bar,false);
      cur=xmacd-sign_xma;
      prev=SignBuffer[bar-1]-XSignBuffer[bar-1];

      //----                  
      SignBuffer[bar]=xmacd;     
      XSignBuffer[bar]=sign_xma;
      XMACDBuffer[bar]=cur*3;
      
      price=PriceSeries(Ma_Price,bar,open,low,high,close);
      curMA=XMA4.XMASeries(1,prev_calculated,rates_total,Ma_Method,Ma_Phase,Ma,price,bar,false);   
            
      int clr=1;
      if(cur>prev && curMA>prevMA) clr=2;
      else if(cur<prev && curMA<prevMA) clr=0;
      ColorXMACDBuffer[bar]=clr;      
      
      if(bar<rates_total-1) prevMA=curMA;
     }
//----     
   return(rates_total);
  }
//+------------------------------------------------------------------+
